#include<stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

void nrerror(char error_text[])
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");

#ifndef _WINDOWS
	exit(-1);
#endif
}

double *vector(int nl, int nh)
{
	double *v;

	v=(double *)calloc((unsigned) (nh-nl+1), sizeof(double));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

double **matrix(int nrl, int nrh, int ncl, int nch)
{
	int i;
	double **m;

	m=(double **) calloc((unsigned) (nrh-nrl+1), sizeof(double*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) calloc((unsigned) (nch-ncl+1), sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}


void free_vector(double *v, int nl, int nh)
{
	free((char*) (v+nl));
}

void free_matrix(double** m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}


//=======================================================================
//#include "stdafx.h"

#include <math.h>

//#include <recipies.h>


#define ROTATE(a,i,j,k,l) g=a[i][j];h=a[k][l];a[i][j]=g-s*(h+g*tau);\
	a[k][l]=h+s*(g-h*tau);

void jacobi(double** a, int n, double d[], double** v, int* nrot)
{
	int j,iq,ip,i;
	double tresh,theta,tau,t,sm,s,h,g,c,*b,*z;

	b=vector(1,n);
	z=vector(1,n);
	for (ip=1;ip<=n;ip++) {
		for (iq=1;iq<=n;iq++) v[ip][iq]=0.0;
		v[ip][ip]=1.0;
	}
	for (ip=1;ip<=n;ip++) {
		b[ip]=d[ip]=a[ip][ip];
		z[ip]=0.0;
	}
	*nrot=0;
	for (i=1;i<=50;i++) {
		sm=0.0;
		for (ip=1;ip<=n-1;ip++) {
			for (iq=ip+1;iq<=n;iq++)
				sm += (double) (fabs(a[ip][iq]));
		}
		if (sm == 0.0) {
			free_vector(z,1,n);
			free_vector(b,1,n);
			return;
		}
		if (i < 4)
			tresh=(double) (0.2*sm/(n*n));
		else
			tresh=0.0;
		for (ip=1;ip<=n-1;ip++) {
			for (iq=ip+1;iq<=n;iq++) {
				g=(double) (100.0*fabs(a[ip][iq]));
				if (i > 4 && (double)(fabs(d[ip])+g) == (double)fabs(d[ip])
					&& (double)(fabs(d[iq])+g) == (double)fabs(d[iq]))
					a[ip][iq]=0.0;
				else if (fabs(a[ip][iq]) > tresh) {
					h=d[iq]-d[ip];
					if ((double)(fabs(h)+g) == (double)fabs(h))
						t=(a[ip][iq])/h;
					else {
						theta=(double) (0.5*h/(a[ip][iq]));
						t=(double) (1.0/(fabs(theta)+sqrt(1.0+theta*theta)));
						if (theta < 0.0) t = -t;
					}
					c=(double) (1.0/sqrt(1+t*t));
					s=t*c;
					tau=(double) (s/(1.0+c));
					h=t*a[ip][iq];
					z[ip] -= h;
					z[iq] += h;
					d[ip] -= h;
					d[iq] += h;
					a[ip][iq]=0.0;
					for (j=1;j<=ip-1;j++) {
						ROTATE(a,j,ip,j,iq)
					}
					for (j=ip+1;j<=iq-1;j++) {
						ROTATE(a,ip,j,j,iq)
					}
					for (j=iq+1;j<=n;j++) {
						ROTATE(a,ip,j,iq,j)
					}
					for (j=1;j<=n;j++) {
						ROTATE(v,j,ip,j,iq)
					}
					++(*nrot);
				}
			}
		}
		for (ip=1;ip<=n;ip++) {
			b[ip] += z[ip];
			d[ip]=b[ip];
			z[ip]=0.0;
		}
	}
	nrerror("Too many iterations in routine JACOBI");
}

#undef ROTATE
//==========================================================
#define  eps 0.0001

static int idMax=0;
static double *** Rotations=NULL;
static double ** MassV=NULL;

double rDiagonal(double did, double Dim,...) 
{ va_list ap;
  int nDim=Dim+eps;
  int id=did+eps;
  double**MassM=matrix(1,Dim,1,Dim);
  int i,j,nrot,fl=0;
   
  va_start(ap,Dim);
  {
    for(i=1;i<=nDim;i++)
    {  MassM[i][i]=va_arg(ap, double);
       for(j=i+1;j<=nDim;j++) MassM[i][j]=MassM[j][i]=va_arg(ap, double);
    }
  } 
  va_end(ap);

  if(idMax==0)
  { Rotations=(double ***) malloc((id+1)*sizeof(double**));
    MassV=(double **) malloc((id+1)*sizeof(double*));
    for(i=0;i<=id;i++) {Rotations[i]=NULL; MassV[i]=NULL;}
    idMax=id;
  }else if(id>idMax)
  { Rotations=(double ***) realloc( Rotations,(id+1)*sizeof(double**));
    MassV=(double **) realloc(MassV, (id+1)*sizeof(double*));
    for(i=idMax+1;i<=id;i++) {Rotations[i]=NULL; MassV[i]=NULL;}
    idMax=id;
  }
  if(Rotations[id]==NULL) 
  { Rotations[id]=matrix(1,Dim,1,Dim); MassV[id]=vector(1,Dim);}
  jacobi(MassM, Dim, MassV[id], Rotations[id], &nrot); 
  free_matrix(MassM,1,Dim,1,Dim);
  do
  	{
		fl=0;
		for(i=1;i<nDim;i++)
			if(fabs(MassV[id][i])>fabs(MassV[id][i+1]))
			{
				double tmp;
				fl=1;
				tmp=MassV[id][i];
				MassV[id][i]=MassV[id][i+1];
				MassV[id][i+1]=tmp;
				for(j=1;j<=nDim;j++)
				{
					tmp=Rotations[id][j][i];
					Rotations[id][j][i]=Rotations[id][j][i+1];
					Rotations[id][j][i+1]=tmp;
				}
			}
	}while(fl);
	
  return did;
}

double rDiagonal2(double did, double Dim,...) 
{ va_list ap;
  int nDim=Dim+eps;
  int id=did+eps;
  double**MassM=matrix(1,nDim,1,nDim), **MassM2=matrix(1,nDim,1,nDim);
  int i,j,nrot,fl=0;
   
  va_start(ap,Dim);
  {
    for(i=1;i<=nDim;i++) for(j=1;j<=nDim;j++)
      MassM[i][j]=va_arg(ap, double);
  } 
  va_end(ap);

  if(idMax==0)
  { Rotations=(double ***) malloc((id+2)*sizeof(double**));
    MassV=(double **) malloc((id+2)*sizeof(double*));
    for(i=0;i<=id+1;i++) {Rotations[i]=NULL; MassV[i]=NULL;}
    idMax=id+1;
  }else if(id>idMax)
  { Rotations=(double ***) realloc( Rotations,(id+2)*sizeof(double**));
    MassV=(double **) realloc(MassV, (id+2)*sizeof(double*));
    for(i=idMax+1;i<=id+1;i++) {Rotations[i]=NULL; MassV[i]=NULL;}
    idMax=id+1;
  }
  if(Rotations[id]==NULL) 
  { Rotations[id]=matrix(1,Dim,1,Dim); MassV[id]=vector(1,Dim);}
  if(Rotations[id+1]==NULL) 
  { Rotations[id+1]=matrix(1,Dim,1,Dim); MassV[id+1]=vector(1,Dim);}
  
  for(i=1;i<=nDim;i++) for(j=1;j<=nDim;j++)
  {
  	int k;
	MassM2[i][j]=0.0;
	for(k=1;k<=nDim;k++)
		MassM2[i][j]+=MassM[i][k]*MassM[j][k];
  }
  jacobi(MassM2, Dim, MassV[id], Rotations[id], &nrot); 
  
  for(i=1;i<=nDim;i++) for(j=1;j<=nDim;j++)
  {
  	int k;
	MassM2[i][j]=0.0;
	for(k=1;k<=nDim;k++)
		MassM2[i][j]+=MassM[k][i]*MassM[k][j];
  }
  jacobi(MassM2, Dim, MassV[id+1], Rotations[id+1], &nrot); 
  free_matrix(MassM,1,Dim,1,Dim);
  free_matrix(MassM2,1,Dim,1,Dim);
 
  do
  	{
		fl=0;
		for(i=1;i<nDim;i++)
			if(fabs(MassV[id][i])>fabs(MassV[id][i+1]))
			{
				double tmp;
				fl=1;
				tmp=MassV[id][i];
				MassV[id][i]=MassV[id][i+1];
				MassV[id][i+1]=tmp;
				for(j=1;j<=nDim;j++)
				{
					tmp=Rotations[id][j][i];
					Rotations[id][j][i]=Rotations[id][j][i+1];
					Rotations[id][j][i+1]=tmp;
					tmp=Rotations[id+1][j][i];
					Rotations[id+1][j][i]=Rotations[id+1][j][i+1];
					Rotations[id+1][j][i+1]=tmp;
				}
			}
	}while(fl);
  
  return did;
}

double MassArray(double did,  double di)
{
  int id=did+eps, i=di+eps;
  return  MassV[id][i];  
}

double MixMatrix(double did,  double di,double dj)
{
  int id=did+eps, i=di+eps,j=dj+eps;
  return  Rotations[id][i][j];  
}



int main(void)
{
  rDiagonal(1.,2.,0.,3.,0.);
  
  printf("masses : %E %E\n", MassArray(1.,1.), MassArray(1.,2.) );
  printf("rotation : %E %E\n", MixMatrix(1.,1.,1.),MixMatrix(1.,1.,2.));
  printf("rotation : %E %E\n", MixMatrix(1.,2.,1.),MixMatrix(1.,2.,2.));

}
